#!/usr/bin/python3
# refer to https://gitlab.freedesktop.org/-/snippets/1665
from gi.repository import GLib, Gio

FPRINT_NAMESPACE = 'net.reactivated.Fprint'
FPRINT_PATH = '/' + FPRINT_NAMESPACE.replace('.', '/')

TARGET_DEVICES_NAMES = [
    'Broadcom Sensors',
]

TIMEOUT = 15000


def is_timeout_error(e):
    return (e.matches(Gio.DBusError.quark(), Gio.DBusError.TIMED_OUT) or
            e.matches(Gio.io_error_quark(), Gio.IOErrorEnum.TIMED_OUT))

i = 0
while i < 7:
    try:
        manager = Gio.DBusProxy.new_for_bus_sync(Gio.BusType.SYSTEM,
                                                 Gio.DBusProxyFlags.NONE, None,
                                                 FPRINT_NAMESPACE,
                                                 FPRINT_PATH + '/Manager',
                                                 FPRINT_NAMESPACE + '.Manager',
                                                 None)
        break
    except GLib.Error as e:
        if not is_timeout_error(e):
            print('[INFO] Get an non timeout error.')
            raise e
        print('[INFO] Waiting for Broadcom fingerprint sensor firmware updating.')
        i += 1
        if i == 7:
            print('[INFO] Firmware updating took more time than expected,\
                 stop waiting and it should be finished in background.')
            exit()

devices_paths = []
devices = []

try:
    [devices_paths] = manager.call_sync('GetDevices', parameters=None,
                                        flags=Gio.DBusCallFlags.NONE,
                                        timeout_msec=TIMEOUT, cancellable=None)
except GLib.Error as e:
    if not is_timeout_error(e):
        raise e
    print('Getting devices took more than expected')

for dev_path in devices_paths:
    dev_proxy = Gio.DBusProxy.new_for_bus_sync(Gio.BusType.SYSTEM,
                                               Gio.DBusProxyFlags.NONE, None,
                                               FPRINT_NAMESPACE, dev_path,
                                               FPRINT_NAMESPACE + '.Device',
                                               None)

    if dev_proxy.get_cached_property('name').unpack() in TARGET_DEVICES_NAMES:
        devices.append(dev_proxy)

for dev in devices:
    # This opens and releases the device, ensuring that it will update the
    # firmware if needed
    print('Found device {}, Claiming it'.
          format(dev.get_cached_property('name')))
    try:
        # Device release will happen once the script completes,
        # leave fprintd handle it
        dev.call_sync('Claim', GLib.Variant('(s)', ['']),
                      Gio.DBusCallFlags.NONE,
                      timeout_msec=TIMEOUT, cancellable=None)
    except GLib.Error as e:
        if not is_timeout_error(e):
            raise e
        print('  Claiming took more than expected')
